ButtonTray = class(Turbine.UI.Control);

local ButtonImages = {
    CreateText = "Thurallor/Blackboard/Images/create_text.tga";
    CreateImage = "Thurallor/Blackboard/Images/create_image.tga";
    Thickness1 = "Thurallor/Blackboard/Images/select_thickness_1.tga";
    Thickness2 = "Thurallor/Blackboard/Images/select_thickness_2.tga";
    Thickness4 = "Thurallor/Blackboard/Images/select_thickness_4.tga";
    CreateGeometric = "Thurallor/Blackboard/Images/create_geometric.tga";
    CreateMap = "Thurallor/Blackboard/Images/create_map.tga";
}

function ButtonTray:Constructor()
    Turbine.UI.Control.Constructor(self);
    self.buttons = {};
    self.buttonOrder = {};
    self.showingButtons = {};
    self.globalButtonStates = {
        ForeColor = Turbine.UI.Color.PaleGoldenrod;
        BackColor = Turbine.UI.Color.Black;
        Thickness = 1;
    };
    
    local function CreateButton(name)
        local button = Turbine.UI.Control();
        button.name = name;
        button:SetBackground(ButtonImages[name]);
        button:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
        button:SetSize(32, 32);
        button.tooltipText = "(Tooltip missing)";
        function button.SetEnabled(b, enabled)
            if (enabled) then
                button.MouseClick = function(b)
                    DoCallbacks(self, "ButtonClicked", name);
                end
            else
                button.MouseClick = nil;
            end
        end
        function button.IsEnabled(b)
            return button.MouseClick;
        end
        button.MouseEnter = function(b)
            if (b:IsEnabled()) then
                b:SetBackColor(Turbine.UI.Color.Goldenrod);
            else
                b:SetBackColor(Turbine.UI.Color.Red);
            end
        end
        button.MouseLeave = function(b)
            b:SetBackColor(Turbine.UI.Color.White);
        end
        button:SetEnabled(true);
        self.buttons[name] = button;
        table.insert(self.buttonOrder, name);
        Thurallor.UI.Tooltip(function(b) return b.tooltipText; end):Attach(button);
        return button;
    end

    local function AddSwatch(button, size)
        local swatch = Turbine.UI.Control();
        swatch:SetParent(button);
        swatch:SetSize(size, size);
        swatch:SetBackColor(Turbine.UI.Color(1, 0.75, 0.75, 0.75));
        swatch.interior = Turbine.UI.Control();
        swatch.interior:SetParent(swatch);
        swatch.interior:SetSize(size - 2, size - 2);
        swatch.interior:SetPosition(1, 1);
        swatch.tooltipText = "(Tooltip missing)";
        swatch.interior.MouseClick = function(swi)
            DoCallbacks(self, "ButtonClicked", swi:GetParent());
        end
        swatch.interior.MouseEnter = function(swi)
            swi:GetParent():SetBackColor(Turbine.UI.Color.Goldenrod);
        end
        swatch.interior.MouseLeave = function(swi)
            swi:GetParent():SetBackColor(Turbine.UI.Color.White);
        end
        function swatch.SetColor(sw, color)
            sw.interior:SetBackColor(color);
        end
        function swatch.GetColor(sw)
            return sw.interior:GetBackColor();
        end
        Thurallor.UI.Tooltip(function(i) return i:GetParent().tooltipText; end):Attach(swatch.interior);
        return swatch;
    end

    local button;

    CreateButton("CreateText");
    CreateButton("CreateImage");
    CreateButton("CreateGeometric");

    button = CreateButton("CreateMap");
    if (not _G.TerrainMap) then
        button:SetEnabled(false);
        button.tooltipText = "TerrainMap is not installed";
    end
    
    button = CreateButton("Thickness");
    button.backgrounds = {
        [1] = ButtonImages.Thickness1,
        [2] = ButtonImages.Thickness2,
        [4] = ButtonImages.Thickness4
    };
    button.SetThickness = function(b, thickness)
        b.thickness = thickness;
        b:SetBackground(b.backgrounds[thickness]);
    end

    button = CreateButton("Colors");
    button.backSwatch = AddSwatch(button, 20);
    button.foreSwatch = AddSwatch(button, 20);
    button.foreSwatch:SetPosition(12, 12);

    button = CreateButton("Color");
    button.swatch = AddSwatch(button, 32);
end

function ButtonTray:SetSize(width, height)
    Turbine.UI.Control.SetSize(self, width, height);
    self:Redraw();
end

function ButtonTray:SetColor(color)
    self.buttons.Color.swatch:SetColor(color);
end

function ButtonTray:GetColor()
    return self.buttons.Color.swatch:GetColor();
end

function ButtonTray:SetForeColor(color)
    self.buttons.Colors.foreSwatch:SetColor(color);
end

function ButtonTray:GetForeColor()
    return self.buttons.Colors.foreSwatch:GetColor();
end

function ButtonTray:SetBackColor(color)
    self.buttons.Colors.backSwatch:SetColor(color);
end

function ButtonTray:GetBackColor()
    return self.buttons.Colors.backSwatch:GetColor();
end

function ButtonTray:SetThickness(thickness)
    return self.buttons.Thickness:SetThickness(thickness);
end

function ButtonTray:GetThickness()
    return self.buttons.Thickness.thickness;
end

function ButtonTray:ButtonClicked(button)
    if ((button == self.buttons.Color.swatch) or (button == self.buttons.Colors.foreSwatch) or (button == self.buttons.Colors.backSwatch)) then
        self:PickColor(button);
    elseif (button == "Thickness") then
        button = self.buttons.Thickness;
        if (button.thickness == 1) then
            button:SetThickness(2);
        elseif (button.thickness == 2) then
            button:SetThickness(4);
        else
            button:SetThickness(1);
        end
        DoCallbacks(self, "ThicknessChanged");
    end
end

function ButtonTray:PickColor(swatch)
    if (self.colorPicker) then
        self.colorPicker:Close();
    end
    
    local colorPicker = Thurallor.UI.ColorPicker(swatch:GetColor());
    local function colorChanged()
        self.currentSwatch:SetColor(self.colorPicker:GetColor());
        DoCallbacks(self, "ColorChanged");
    end
    AddCallback(colorPicker, "ColorChanged", colorChanged);
    AddCallback(colorPicker, "Accepted", colorChanged);
    AddCallback(colorPicker, "Canceled", function()
        self.colorPicker = nil;
    end);
    
    self.colorPicker = colorPicker;
    self.currentSwatch = swatch;
end

function ButtonTray:SetContext(context, shapes)
--Puts("ButtonTray:SetContext " .. Serialize({context, shapeTypes}));
    self.shape = nil;
    self.context = context;
    self.showingButtons = {};
    
    local function showButton(name, tooltipText)
        local button = self.buttons[name];
        if (button) then
            self.showingButtons[name] = true;
            if (button:IsEnabled()) then
                self.buttons[name].tooltipText = tooltipText;
            end
        end
    end
    
    if (context == "Global") then
        showButton("CreateText", "Create a new text shape");
        showButton("CreateImage", "Create a new image shape");
        showButton("CreateGeometric", "Create a new geometric shape");
        showButton("CreateMap", "Create a new radar map shape");
        showButton("Thickness", "Set the default line thickness for new geometric shapes");
        showButton("Colors", "Set the default colors for new shapes");
        self.buttons.Colors.foreSwatch.tooltipText = "Set the default foreground color for new shapes";
        self.buttons.Colors.backSwatch.tooltipText = "Set the default background color for new shapes";
    elseif (context == "Shape") then
        local shapeClass = next(shapes);
        self.showingButtons = {};
        if (next(shapes, shapeClass)) then
            -- Multiple types of shape selected.  Hide all buttons.
            -- To do: Handle selection of multiple shape types: show only the properties that apply to all of them.
        else
            if (#shapes[shapeClass] > 1) then
                -- Multiple shapes of this type selected.  Hide all buttons.
                -- To do: Handle selection of multiple shapes of the same type
            else
                self.shape = shapes[shapeClass][1];
                if (shapeClass == TextShape) then
                    showButton("Color", "Change the text color for the selected shape");
                    self.buttons.Color.swatch.tooltipText = self.buttons.Color.tooltipText;
                    showButton("Font", "Change the text font for the selected shape");
                    showButton("HorizontalAlignment", "Change the horizontal text alignment for the selected shape");
                    showButton("VerticalAlignment", "Change the vertical text alignment for the selected shape");
                elseif (shapeClass == RectangleShape) then
                    showButton("Thickness", "Change the outline thickness for the selected shape");
                    showButton("Colors", "Change the colors for the selected shape");
                    self.buttons.Colors.foreSwatch.tooltipText = "Change the outline color for the selected shape";
                    self.buttons.Colors.backSwatch.tooltipText = "Change the interior color for the selected shape";
                end
            end
        end
    end

    -- If no shapes selected, attach button states to self.globalButtonStates
    if (not self.shape) then
        self:SetForeColor(self.globalButtonStates.ForeColor);
        self:SetBackColor(self.globalButtonStates.BackColor);
        self:SetThickness(self.globalButtonStates.Thickness);
        self.ColorChanged = function()
            self.globalButtonStates.ForeColor = self:GetForeColor();
            self.globalButtonStates.BackColor = self:GetBackColor();
        end
        self.ThicknessChanged = function()
            self.globalButtonStates.Thickness = self:GetThickness();
        end
    else
        -- A single shape is selected.  Attach button states to shape properties
        if (self.showingButtons.Color) then
            self:SetColor(self.shape:GetColor());
            self.ColorChanged = function()
                self.shape:SetColor(self:GetColor());
            end
        elseif (self.showingButtons.Colors) then
            self:SetForeColor(self.shape:GetForeColor());
            self:SetBackColor(self.shape:GetBackColor());
            self.ColorChanged = function()
                self.shape:SetForeColor(self:GetForeColor());
                self.shape:SetBackColor(self:GetBackColor());
            end
        end
        if (self.showingButtons.Thickness) then
            self:SetThickness(self.shape:GetOutlineSize());
            self.ThicknessChanged = function()
                self.shape:SetOutlineSize(self:GetThickness());
            end
        end
    end
    self:Redraw();
end

function ButtonTray:Redraw()
    self:GetControls():Clear();
    local spacing = 4;
    local width, height = self:GetSize();

    local container = Turbine.UI.Control();
    container:SetParent(self);
    container:SetSize(width, height);

    local button, swatch;
    local top = 0;
    for _, name in ipairs(self.buttonOrder) do
        if (self.showingButtons[name]) then
            local button = self.buttons[name];
            button:SetParent(container);
            button:SetPosition(0, top);
            top = top + button:GetHeight() + spacing;
        end
    end
    
    -- Center the container vertically
    top = math.max(0, top - spacing);
    container:SetHeight(top);
    container:SetTop(math.floor(0.5 + (height - top) / 2));
end
