Group = class(Shape);

function Group:Constructor(baseClassName)
    baseClassName = baseClassName or "Turbine.UI.Control";
    Shape.Constructor(self, baseClassName);
    self.idStr = "new Group";
    
    self.shapesByIDStr = {};
    self.changedShapes = {};
    self.lastShapeID = 0;
    self.minZOrder = 0;
    self.maxZOrder = -1;

    -- Hardcoded properties
    self.properties.Class = Group;
    self.changedProperties.Class = Group;
    self.resizable = false;

    -- Default property values
    self.applyingRemoteChanges = true;
    self:SetSize(100, 100);
    self.applyingRemoteChanges = false;
    
    self:SavePropertyDefaults();
end

function Group:GetNextShapeID()
    local idStr;
    repeat
        self.lastShapeID = self.lastShapeID + 1;
        idStr = Shape.CreateIDStr(self.playerName, self.lastShapeID);
    until (not self.shapesByIDStr[idStr]);
    return self.lastShapeID;
end

function Group:AddShapeAt(shape, left, top)
    local idStr = shape:GetIDStr();
--Puts("  self.shapesByIDStr[" .. idStr .. "] := " .. tostring(shape));
    self.shapesByIDStr[idStr] = shape;
    shape:SetParent(self);
    self:BringShapeToFront(shape);

    AddCallback(shape, "Changed", function(s)
        self:ShapeChanged(s);
    end);

    if (left) then
        shape:SetPosition(left, top);
    end
end

function Group:AddShape(shape)
end

function Group:RemoveShape(shape)
    local idStr = shape:GetIDStr();
    self.shapesByIDStr[idStr] = nil;
    shape:SetParent(nil);
end

function Group:CloneShape(shape)
    local newShape = shape:GetClone(self:GetNextShapeID(), self.playerName);
    local left, top = shape:GetPosition();
    self:AddShapeAt(newShape, left + 2, top + 2);
end

function Group:ShapeChanged(shape)
    self.changedShapes[shape] = true;
--Puts("Shape " .. shape:GetIDStr() .. " changed");    
    DoCallbacks(self, "Changed");
    if (shape:IsDeleted()) then
        self:RemoveShape(shape);
    end
end

function Group:GetChanges()
    local str = Shape.GetChanges(self);
    for shape, _ in pairs(self.changedShapes) do
        if (shape:IsChanged()) then
            local changes = shape:GetChanges();
            str = str .. Shape.encoder:Encode({ Shape = changes });
        end
    end
    return str;
end

function Group:ResetChanges()
--Puts("[Group " .. self:GetIDStr() .. "] resetting changes");
    Shape.ResetChanges(self);
    for shape, _ in pairs(self.changedShapes) do
        shape:ResetChanges();
    end
    self.changedShapes = {};
end

-- only called from within ApplyChanges()
function Group:SetShape(value, from)
    if (type(value) == "table") then
        for _, str in ipairs(value) do
            self:_SetShape(str, from);
        end
    else
        self:_SetShape(value, from);
    end
end

-- only called from within SetShape()
function Group:_SetShape(str, from)
    local properties = Shape.encoder:Decode(str);
    properties.Owner = properties.Owner or from;
    local idStr = Shape.CreateIDStr(properties.Owner, properties.ID);
    local shape = self.shapesByIDStr[idStr];
    if (not shape) then
        shape = properties.Class();
        shape:SetOwner(properties.Owner);
        shape:SetID(properties.ID);
        self:AddShapeAt(shape);
        shape:ResetChanges();
    end
    properties.Class = nil;
    shape:ApplyChanges(properties, from);
end

function Group:IsChanged() 
    return next(self.changedProperties) or next(self.changedShapes);
end

function Group:Serialize(shapesOnly)
    local str = "";
    if (not shapesOnly) then
        str = Shape.Serialize(self);
    end
    for _, shape in pairs(self.shapesByIDStr) do
        local shapeStr = shape:Serialize();
        str = str .. Shape.encoder:Encode({ Shape = shapeStr });
    end
    return str;
end

function Group:ExportShapes()
    return self:Serialize(true);
end

function Group:ImportShapes(str)
    local shapes = Shape.encoder:Decode(str);
    for n, shapeStr in pairs(shapes.Shape) do
        Puts("Shape" .. n .. ": " .. shapeStr);
        local properties = Shape.encoder:Decode(shapeStr);
        local shape = properties.Class();
        shape:SetOwner(self.playerName);
        shape:SetID(self:GetNextShapeID());
        self:AddShapeAt(shape);
        properties.Class = nil;
        properties.Owner = nil;
        properties.ID = nil;
        shape:ApplyChanges(properties, self.playerName);
    end
    DoCallbacks(self, "Changed"); -- tell editor to update sendButton
end

function Group:GetMaxZOrder()
    local maxZOrder = -2147483648
    for _, shape in pairs (self.shapesByIDStr) do
        maxZOrder = math.max(maxZOrder, shape:GetZOrder());
    end
    return maxZOrder;
end

function Group:GetMinZOrder()
    local minZOrder = 2147483647;
    for _, shape in pairs (self.shapesByIDStr) do
        minZOrder = math.min(minZOrder, shape:GetZOrder());
    end
    return minZOrder;
end

function Group:IsShapeInFront(shape)
    return (shape:GetZOrder() == self:GetMaxZOrder());
end

function Group:IsShapeInBack(shape)
    return (shape:GetZOrder() == self:GetMinZOrder());
end

function Group:BringShapeToFront(shape)
    shape:SetZOrder(self:GetMaxZOrder() + 1);
    shape:SetParent(nil);
    shape:SetParent(self);
end

function Group:SendShapeToBack(shape)
    shape:SetZOrder(self:GetMinZOrder() - 1);
    shape:SetParent(nil);
    shape:SetParent(self);
end
