RadarMapShape = class(Group);

function RadarMapShape:Constructor()
    Group.Constructor(self, "Turbine.UI.Label");
    self.idStr = "new RadarMapShape";
    self.tiles = {};

    self.pin = Turbine.UI.Control();
    self.pin:SetBackground(0x41005e89);
    self.pin:SetBlendMode(Turbine.UI.BlendMode.Overlay);
    self.pin:SetSize(16, 16);
    self.pin:SetZOrder(2);

    -- Hardcoded property values
    self.properties.Class = RadarMapShape;
    self.changedProperties.Class = RadarMapShape;
    self.baseClass.SetMultiline(self, true);
    self.baseClass.SetForeColor(self, Turbine.UI.Color.White);
    self.baseClass.SetFont(self, Turbine.UI.Lotro.Font.Verdana16);
    self.baseClass.SetFontStyle(self, Turbine.UI.FontStyle.Outline);
    self.baseClass.SetTextAlignment(self, Turbine.UI.ContentAlignment.MiddleCenter);
    self.baseClass.SetOutlineColor(self, Turbine.UI.Color(0.75, 0, 0, 0));
    self.properties.XOffset = 0;
    self.properties.YOffset = 0;
    self.resizable = _G.TerrainMap;
    
    -- Default property values
    self.applyingRemoteChanges = true;
    self:SetSize(200, 200);
    self:SetLocation("r1 lx938 ly948 ox58.53 oy86.61 oz388.70 h90.0"); -- West Bree
--    self:SetLocation("r2 lx837 ly979 ox106.40 oy72.58 oz457.88"); -- Stangard
--    self:SetLocation("r4 lx1462 ly831 ox126.64 oy157.16 oz107.57 h185.6") -- Udun Foothold
--    self:SetLocation("r3 lx755 ly497 ox66.74 oy24.68 oz240.06 h180.0"); -- Gondor
    self:SetPinEnabled(true);
    self.applyingRemoteChanges = false;
    
    self:SavePropertyDefaults();
end

--function RadarMapShape:SetWantsUpdates(wants)
--    Puts(Turbine.Engine.GetCallStack());
--    Turbine.UI.Label.SetWantsUpdates(self, wants);
--end

function RadarMapShape:_SizeChanged()
    self:SetWantsUpdates(true); -- needs redraw
    Shape._SizeChanged(self);
end

function RadarMapShape:SetProperty(name, value)
    Shape.SetProperty(self, name, value);
end

-- This conversion function is largely copied from the Waypoint plugin.  Thanks, LunarWtr!
function RadarMapShape:_ParseLocation(location)
	for _, regexp in ipairs({
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
        -- not likely to match last --
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)'
    }) do
        local r, lx, ly, ox, oy, oz, h = location:match(regexp);
        h = h or 0;
        if (r) then
            lx, ly, ox, oy = tonumber(lx), tonumber(ly), tonumber(ox), tonumber(oy);
            local x = (( math.floor( lx / 8 ) * 160 + ox ) - 29360) / 200;
            local y = (( math.floor( ly / 8 ) * 160 + oy ) - 24880) / 200;
            local coords = string.format("%.2f", math.abs(y)) .. (((y < 0) and "S") or "N") .. ", "
                .. string.format("%.2f", math.abs(x)) .. (((x < 0) and "W") or "E");
            local heading = round(h, 0) .. "°";
            return coords, heading, tonumber(r), x, y, h;
        end
	end
end

function RadarMapShape:_CreateTile(x, y)
    local tile = Turbine.UI.Control();
--local tile = Turbine.UI.Label();
    tile:SetSize(200, 200);
    tile:SetParent(self);
    tile:SetMouseVisible(false);
    tile:SetPosition(x, y);
    return tile;
end

-- Creates and positions the tiles, but doesn't set their backgrounds.
function RadarMapShape:_CreateTiles(xOffset, yOffset)
    local width, height = self:GetSize();
    local centerX, centerY = math.floor(0.5 + width / 2), math.floor (0.5 + height / 2);
    
    local centerTileLeft = centerX - xOffset;
    local centerTileRight = centerTileLeft + 200;
    local tilesLeft = math.ceil(math.max(0, centerTileLeft) / 200);
    local tilesRight = math.ceil(math.max(0, width - centerTileRight) / 200);

    local centerTileTop = centerY - yOffset;
    local centerTileBottom = centerTileTop + 200;
    local tilesAbove = math.ceil(math.max(0, centerTileTop) / 200);
    local tilesBelow = math.ceil(math.max(0, height - centerTileBottom) / 200);
    
    local xTiles = tilesLeft + 1 + tilesRight;
    local yTiles = tilesAbove + 1 + tilesBelow;
    local top = centerTileTop - tilesAbove * 200;

    self.tiles = {};
    for yTile = 1, yTiles do
        local left = centerTileLeft - tilesLeft * 200;
        for xTile = 1, xTiles do
            table.insert(self.tiles, self:_CreateTile(left, top));
            left = left + 200;
        end
        top = top + 200;
    end
end

-- Gets the backgrounds from TerrainMap for the already-positioned tiles
function RadarMapShape:_GetAssetIDs(region, centerNorthSouth, centerEastWest)
    local width, height = self:GetSize();
    local centerX, centerY = math.floor(0.5 + width / 2), math.floor (0.5 + height / 2);
    local assetIDs = {};
    for _, tile in ipairs(self.tiles) do
        local left, top = tile:GetPosition();
        local xDistance = centerX - left;
        local yDistance = centerY - top;
        local eastWest = centerEastWest - xDistance * (0.8 / 200);
        local northSouth = centerNorthSouth + yDistance * (0.8 / 200);
        local assetID = _G.TerrainMap.GetTile(region, northSouth - 0.4, eastWest + 0.4);
        -- 0x41120573 is the average of the asset IDs of all terrain tiles, so we store only the delta from this to minimize transmission size
        -- 0x41120573 is not itself a valid asset ID, so we can use 0 as a special value indicating nil
        if (type(assetID) ~= "number") then
            -- Don't bother with JPGs, since they can't be transmitted
            assetID = 0;
        else
            assetID = assetID - 0x41120573;
        end
        table.insert(assetIDs, assetID);
    end
    return assetIDs;
end

function RadarMapShape:_ApplyBackgrounds(assetIDs)
    for n, tile in ipairs(self.tiles) do
        local assetID = assetIDs[n];
        if (assetID == 0) then
            assetID = nil;
        else
            assetID = assetID + 0x41120573;
        end
        tile:SetBackground(assetID);
    end
end

function RadarMapShape:Update()
    self:SetWantsUpdates(false);

    self:GetControls():Clear();
    local _, _, region, eastWest, northSouth, _ = self:_ParseLocation(self.properties.Location);
    if (not region) then
        self.baseClass.SetText(self, "No terrain data available for this location");
        return;
    else
        self.baseClass.SetText(self, "");
    end

    local xOffset, yOffset = self.properties.XOffset, self.properties.YOffset;
    if (_G.TerrainMap) then
        _, xOffset, yOffset = _G.TerrainMap.GetTile(region, northSouth, eastWest);
        self:SetProperty("XOffset", xOffset);
        self:SetProperty("YOffset", yOffset);
    end
    self:_CreateTiles(xOffset, yOffset);
    
    local assetIDs = self.properties.Backgrounds;
    if (_G.TerrainMap) then
        assetIDs = self:_GetAssetIDs(region, northSouth, eastWest);
        self:SetProperty("Backgrounds", assetIDs);
    end
    self:_ApplyBackgrounds(assetIDs);
    
    self.pin:SetParent(nil);
    if (self.properties.PinEnabled) then
        local width, height = self:GetSize();
        local x, y = math.floor(0.5 + width / 2), math.floor(0.5 + height / 2);
        self.pin:SetParent(self);
        self.pin:SetPosition(x - 8, y - 8);
    end
end

-- Custom tracked properties
function RadarMapShape:SetLocation(location)
    self:SetProperty("Location", location);
    self:SetWantsUpdates(true); -- needs redraw
end

function RadarMapShape:GetLocation()
    return self.properties.Location;
end

function RadarMapShape:SetBackgrounds(assetIDs)
    self:SetProperty("Backgrounds", assetIDs);
    self:SetWantsUpdates(true); -- needs redraw
end

function RadarMapShape:GetBackgrounds()
    return self.properties.Backgrounds;
end

function RadarMapShape:SetPinEnabled(enabled)
    self:SetProperty("PinEnabled", enabled);
    self:SetWantsUpdates(true); -- needs redraw
end

function RadarMapShape:GetPinEnabled()
    return self.properties.PinEnabled;
end

function RadarMapShape:SetXOffset(xOffset)
    self:SetProperty("YOffset", yOffset);
    self:SetWantsUpdates(true); -- needs redraw
end

function RadarMapShape:GetXOffset()
    return self.properties.XOffset;
end

function RadarMapShape:SetYOffset(yOffset)
    self:SetProperty("YOffset", yOffset);
    self:SetWantsUpdates(true); -- needs redraw
end

function RadarMapShape:GetYOffset()
    return self.properties.YOffset;
end

-- Standard tracked properties
function RadarMapShape:SetWidth(width)
    Shape.SetWidth(self, width);
    self:SetWantsUpdates(true); -- needs redraw
end

function RadarMapShape:SetHeight(height)
    Shape.SetHeight(self, height);
    self:SetWantsUpdates(true); -- needs redraw
end

