RectangleShape = class(Shape);

function RectangleShape:Constructor()
    Shape.Constructor(self, "Turbine.UI.Control");
    self.idStr = "new RectangleShape";

    self.borders = {};
    for _, side in pairs ({ "left", "top", "right", "bottom" }) do
        local control = Turbine.UI.Control();
        control:SetParent(self);
        control:SetMouseVisible(false);
        self.borders[side] = control;
    end

    self.interior = Turbine.UI.Control();
    self.interior:SetParent(self);
    self.interior:SetMouseVisible(false);

    -- Hardcoded property values
    self.properties.Class = RectangleShape;
    self.changedProperties.Class = RectangleShape;
    self.resizable = true;
    
    -- Default property values
    self.applyingRemoteChanges = true;
    self:SetOutlineSize(1);
    self:SetSize(100, 100);
    self:SetForeColor(Turbine.UI.Color.White);
    self:SetBackColor(Turbine.UI.Color.Black);
    self:SetHollow(false);
    self.applyingRemoteChanges = false;
    
    self:SavePropertyDefaults();
end

function RectangleShape:_SizeChanged()
    self:_Redraw();
    Shape._SizeChanged(self);
end

function RectangleShape:_Redraw()
    local thickness = self.properties.OutlineSize;
    local width, height = self:GetSize();
    self.borders.left:SetSize(thickness, height);
    self.borders.top:SetSize(width, thickness);
    self.borders.right:SetLeft(width - thickness);
    self.borders.right:SetSize(thickness, height);
    self.borders.bottom:SetTop(height - thickness);
    self.borders.bottom:SetSize(width, thickness);
    if (self.properties.Hollow) then
        self.interior:SetParent(nil);
    else
        self.interior:SetParent(self);
        self.interior:SetPosition(thickness, thickness);
        self.interior:SetSize(width - (2 * thickness), height - (2 * thickness));
    end
end

-- Custom tracked properties
function RectangleShape:SetOutlineSize(thickness)
    self:SetProperty("OutlineSize", thickness);
    self:_Redraw();
end

function RectangleShape:GetOutlineSize()
    return self.properties.OutlineSize;
end

function RectangleShape:SetHollow(hollow)
    self:SetProperty("Hollow", hollow);
    self:_Redraw();
end

function RectangleShape:IsHollow()
    return self.properties.Hollow;
end

-- Standard tracked properties
function RectangleShape:SetForeColor(color)
    for _, side in pairs ({ "left", "top", "right", "bottom" }) do
        self.borders[side]:SetBackColor(color);
    end
    self:SetProperty("ForeColor", color);
end

function RectangleShape:GetForeColor()
    return self.properties.ForeColor;
end

function RectangleShape:SetBackColor(color)
    self.interior:SetBackColor(color);
    self:SetProperty("BackColor", color);
end

function RectangleShape:GetBackColor()
    return self.properties.BackColor;
end
