SendButton = class(Turbine.UI.Control);

function SendButton:Constructor()
    Turbine.UI.Control.Constructor(self);
    self:SetSize(32, 32);

    self.overlay = Turbine.UI.Control();
    self.overlay:SetSize(32, 32);
    self.overlay:SetParent(self);
    self.overlay:SetBackground("/Thurallor/Blackboard/Images/send.tga");
    self.overlay:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
    
    local label = Turbine.UI.Label();
    label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
    label:SetFont(Turbine.UI.Lotro.Font.Verdana20);
    label:SetFontStyle(Turbine.UI.FontStyle.Outline);
    label:SetMultiline(false);
    label:SetMouseVisible(false);
    label:SetParent(self);
    label:SetSize(30, 20);
    label:SetTop(12);
    self.number = label;

    self.maxPacketSize = 381;
--self.maxPacketSize = 10;
    self.command = "/8";
    self:SetData(nil);
end

function SendButton:SetCommand(command)
    self.command = command;
    self:UpdateShortcut();
end

function SendButton:SetData(data)
    data = data or "";
    self.packets = {};
    local numPackets = math.ceil(data:len() / self.maxPacketSize);
    for c = numPackets, 1, -1 do
        length = math.min(self.maxPacketSize, data:len());
        local packet = "<u>B</u>" .. string.format("%02d", c - 1) .. data:sub(1, length);
        data = data:sub(length + 1);
        table.insert(self.packets, packet);
    end
--Puts("packets = " .. Serialize(self.packets));
    self:UpdateClicksNeeded();
    self:UpdateShortcut();
end

function SendButton:UpdateClicksNeeded()
    local packetsLeft = #self.packets;
    self.number:SetText(packetsLeft);
    self:SetEnabled(packetsLeft > 0);
end

function SendButton:UpdateShortcut()
    if (self.quickslot) then
        self.quickslot:SetParent(nil);
    end
    if (#self.packets > 0) then
        local packet = table.remove(self.packets, 1);
        local quickslot = Turbine.UI.Lotro.Quickslot();
        quickslot:SetParent(self);
        quickslot:SetSize(32, 32);
        quickslot:SetUseOnRightClick(true);
        quickslot.MouseClick = function(_, args)
            self:ShortcutClicked(args);
        end;
        local alias = self.command .. " " .. packet;
        local shortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, alias);
        quickslot:SetShortcut(shortcut);
        self.quickslot = quickslot;
    end
end

function SendButton:ShortcutClicked(args)
    if (not self.transferStarted) then
        DoCallbacks(self, "TransferStarted");
        self.transferStarted = true;
    end
    if (#self.packets == 0) then
        DoCallbacks(self, "TransferComplete");
        self.transferStarted = false;
    end
    -- self.quickslot:IsUseOnRightClick() is true, so we don't care which button was clicked
    self:UpdateClicksNeeded();
    self:UpdateShortcut();
end

function SendButton:SetEnabled(enabled)
    self.enabled = enabled;
    self:UpdateColor();
    self.overlay:SetMouseVisible(not enabled);
    self.number:SetVisible(enabled);
end

function SendButton:SetForeColor(color)
    self.number:SetForeColor(color);
end

function SendButton:SetBackColor(color)
    self.overlay:SetBackColor(color);
end

function SendButton:GetForeColor(color)
    return self.number:GetForeColor();
end

function SendButton:GetBackColor()
    return self.overlay:GetBackColor();
end

function SendButton:UpdateColor()
    local color;
    if (self.enabled) then
        self.overlay:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
        if (self.mouseInside) then
            self:SetForeColor(Turbine.UI.Color.Goldenrod);
            self:SetBackColor(Turbine.UI.Color.Goldenrod);
        else
            self:SetForeColor(Turbine.UI.Color.White);
            self:SetBackColor(Turbine.UI.Color.White);
        end
    else
        self:SetForeColor(Turbine.UI.Color.White);
        self:SetBackColor(Turbine.UI.Color.White);
        self.overlay:SetBlendMode(Turbine.UI.BlendMode.Overlay);
    end
end

function SendButton:MouseEnter()
    self.mouseInside = true;
    self:UpdateColor();
end

function SendButton:MouseLeave()
    self.mouseInside = false;
    self:UpdateColor();
end
