Shape = class(Turbine.UI.Control);

function Shape:Constructor(baseClassName)
    self.baseClass = loadstring("return " .. baseClassName)();
    self.baseClass.Constructor(self);
    self.properties = { Class = Shape };
    self.changedProperties = { Class = Shape };
    self.playerName = Turbine.Gameplay.LocalPlayer:GetInstance():GetName();
    self.idStr = "new Shape";

    -- Add internal handlers for some events; these handlers also need to be called separately
    AddCallback(self, "SizeChanged", function()
        self:_SizeChanged();
    end);
    AddCallback(self, "PositionChanged", function()
        self:_PositionChanged();
    end);
    
    self:SavePropertyDefaults();
end

-- Set Mouse action callbacks for moving and resizing
function Shape:MouseEnter()
    self.mouseInside = true;
    self:SetHighlighted(true);
end

function Shape:MouseDown(args)
    if (args.Button == Turbine.UI.MouseButton.Left) then
        self.leftMouseDown = true;
        self.originalLeft, self.originalTop = self:GetPosition();
        self.originalMouseX, self.originalMouseY = Turbine.UI.Display.GetMousePosition();
        self.mouseMovedAfterMouseDown = false;
    end
end

function Shape:MouseUp(args)
    if (args.Button == Turbine.UI.MouseButton.Left) then
        self.leftMouseDown = false;
    elseif (args.Button == Turbine.UI.MouseButton.Right) then
        self:ShowContextMenu();
    end
end

function Shape:MouseLeave()
    self.mouseInside = false;
    self:SetHighlighted(false);
end

function Shape:_SizeChanged()
    local width, height = self:GetSize();
    if (self.highlight) then
        self.highlight:SetSize(width, height);
    end
    if (self.marquee) then
        self.marquee:SetSize(width + 2, height + 2);
    end
    if (self.resizer) then
        local left, top = self:GetPosition();
        self.resizer:SetPosition(left + width, top + height);
    end
end

function Shape:_PositionChanged()
    local left, top = self:GetPosition();
    if (self.highlight) then
        self.highlight:SetPosition(left, top);
    end
    if (self.marquee) then
        self.marquee:SetPosition(left - 1, top - 1);
    end
    if (self.resizer) then
        local width, height = self:GetSize();
        self.resizer:SetPosition(left + width, top + height);
    end
end

function Shape:MouseMove()
    if (self.leftMouseDown) then
        local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
        local xShift, yShift = mouseX - self.originalMouseX, mouseY - self.originalMouseY;
        local newLeft, newTop = self.originalLeft + xShift, self.originalTop + yShift;
        self:SetPosition(newLeft, newTop);
        self.mouseMovedAfterMouseDown = true;
    end
end

-- Keep track of property changes so they can be sent as updates, or undone
function Shape:SetProperty(property, value)
--Puts("[Shape " .. self:GetIDStr() .. "]." .. tostring(property) .. " := " .. tostring(value));
    if (self.applyingRemoteChanges) then
        self.properties[property] = value;
        self.changedProperties[property] = nil;
    elseif (value ~= self.properties[property]) then
        self.properties[property] = value;
        self.changedProperties[property] = value;
        DoCallbacks(self, "Changed");
    end
end

function Shape:GetProperty(property)
    return self.properties[property];
end

function Shape:IsChanged()
    return next(self.changedProperties);

end

function Shape:GetChanges()
    return self:GetIDStr() .. Shape.encoder:Encode(self.changedProperties);
end

function Shape:ResetChanges()
--Puts("[Shape " .. self:GetIDStr() .. "] reset changes");
    self.changedProperties = {};
end

function Shape:ApplyChanges(properties, from)
--Puts("Shape: ApplyChanges: " .. Serialize(properties));
    if (from ~= self.playerName) then
        self.applyingRemoteChanges = true;
    end
    for name, value in pairs(properties) do
--Puts("   Applying " .. name .. " = " .. Serialize(value));    
        local setFunc = self["Set" .. name];
        setFunc(self, value, from);
    end
    self.applyingRemoteChanges = false;
--Puts("Finished applying changes");
end

function Shape:Serialize()
    local str = "";
    for name, value in pairs(self.properties) do
        if (self.properties[name] ~= self.propertyDefaults[name]) then
            str = str .. Shape.encoder:Encode({ [name] = value });
        end
    end
    return str;
end

function Shape:SavePropertyDefaults()
    self.propertyDefaults = {};
    self.propertyDefaults = ShallowTableCopy(self.properties);
    self.propertyDefaults.Class = nil;
end


function Shape:ShowContextMenu()
    if (self:IsSelected()) then
        self:GetParent():ShowSelectionContextMenu();
    else
        self:GetContextMenu():ShowMenu();
    end
end

function Shape:GetContextMenu()
    --to do: handle permissions
    local menu, items, item;
    menu = Turbine.UI.ContextMenu();
    items = menu:GetItems();

    local inFront = self:IsInFront();
    item = Turbine.UI.MenuItem("Bring to front", not inFront, inFront);
    item.Click = function()
        self:BringToFront();
    end
    items:Add(item);
    
    local inBack = self:IsInBack();
    item = Turbine.UI.MenuItem("Send to back", not inBack, inBack);
    item.Click = function()
        self:SendToBack();
    end
    items:Add(item);

    item = Turbine.UI.MenuItem("Clone this shape");
    item.Click = function()
        self:GetParent():CloneShape(self);
    end
    items:Add(item);

    item = Turbine.UI.MenuItem("Delete this shape");
    item.Click = function()
        self:SetDeleted(true);
    end
    items:Add(item);
    
    return menu;
end

function Shape:GetClone(newID, newOwner)
    local newShape = self.properties.Class();
    -- Copy all properties to new shape
    for name, value in pairs(self.properties) do
        if (name ~= "Class") then
if (type(value) == "table") then
Puts("Warning: New shape references old shape's table value for " .. name);
end    
            local setFunc = newShape["Set" .. name];
            setFunc(newShape, value);
        end
    end
    newShape:SetID(newID);
    newShape:SetOwner(newOwner);
    return newShape;
end

function Shape:BringToFront()
    return self:GetParent():BringShapeToFront(self);
end

function Shape:SendToBack()
    return self:GetParent():SendShapeToBack(self);
end

function Shape:Delete()
    self:GetParent():RemoveShape(self);
end

function Shape:IsInFront()
    return self:GetParent():IsShapeInFront(self);
end

function Shape:IsInBack()
    return self:GetParent():IsShapeInBack(self);
end

-- Custom tracked properties
function Shape:SetDeleted(deleted)
    if (deleted) then
        self.changedProperties = {};
        self:SetProperty("Deleted", true);
    else
        -- to do: support undeleting?
    end
end

function Shape:IsDeleted()
    return self.properties.Deleted;
end

function Shape:SetID(id)
    -- normally should only get set once; don't track changes
    self.properties.ID = id;
    self:UpdateIDStr();
end

function Shape:GetID()
    return self.properties["ID"];
end

function Shape:SetOwner(owner)
    -- normally should only get set once; don't track changes
    self.properties.Owner = owner;
    self:UpdateIDStr();
end

function Shape:GetOwner()
    return self.properties.Owner;
end

function Shape.CreateIDStr(owner, id)
    local playerName = Turbine.Gameplay.LocalPlayer.GetInstance():GetName();
    local str = Shape.encoder:Encode({ ID = id }); 
    if (owner ~= playerName) then
        str = Shape.encoder:Encode({ Owner = owner }) .. str;
    end
    return str;
end

function Shape:UpdateIDStr()
    self.idStr = Shape.CreateIDStr(self.properties.Owner, self.properties.ID);
end

function Shape:GetIDStr()
    return self.idStr;
end

function Shape:SetSelected(selected)
    self:SetProperty("Selected", selected);
    if (self.marquee) then
        self.marquee:SetParent(nil);
        self.marquee = nil;
    end
    if (self.resizer) then
        self.resizer:SetParent(nil);
        self.resizer = nil;
    end
    if (selected) then
        self.marquee = Thurallor.UI.Marquee();
        self.marquee:SetParent(self:GetParent());
        self.marquee:SetZOrder(2147483647);
        self.marquee:SetVisible(true);
        if (self.resizable) then
            self.resizer = GrabPoint();
            self.resizer:SetParent(self:GetParent());
            AddCallback(self.resizer, "Drag", function(_, args)
                local width, height = self:GetSize();
                width = math.max(5, width + args.x);
                height = math.max(5, height + args.y);
                self:SetSize(width, height);
            end);
        end
        self:_SizeChanged();
        self:_PositionChanged();
    end
    if (selected) then
        self.baseClass.Focus(self);
    end
end

function Shape:IsSelected()
    return self.properties["Selected"];
end

-- Custom untracked properties
function Shape:SetHighlighted(highlighted)
    if (self.highlight) then
        self.highlight:SetParent(nil);
        self.highlight = nil;
    end
    if (highlighted) then
        self.highlight = Turbine.UI.Control();
        self.highlight:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
        self.highlight:SetBackColor(Turbine.UI.Color(0.25, 1, 1, 1));
        self.highlight:SetMouseVisible(false);
        self.highlight:SetParent(self:GetParent());
        self:_SizeChanged();
        self:_PositionChanged();
    end
end

function Shape:SetResizable(resizable)
    if (resizable ~= self.resizable) then
        self.resizable = resizable;
        self:SetSelected(self.properties.Selected); -- redraw with or without GrabPoint
    end
end

function Shape:GetClass()
    return self.properties.Class;
end

-- Standard tracked properties
function Shape:SetSize(width, height)
    self:SetWidth(width);
    self:SetHeight(height);
    DoCallbacks(self, "SizeChanged");
end

function Shape:SetWidth(width)
    self.baseClass.SetWidth(self, width);
    self:SetProperty("Width", width);
    if (self.overlay) then
        self.overlay:SetWidth(width);
    end
end

function Shape:SetHeight(height)
    self.baseClass.SetHeight(self, height);
    self:SetProperty("Height", height);
    if (self.overlay) then
        self.overlay:SetHeight(height);
    end
end

function Shape:SetPosition(left, top)
    self.baseClass.SetPosition(self, left, top);
    self:SetProperty("Left", left);
    self:SetProperty("Top", top);
    if (self.overlay) then
        self.overlay:SetPosition(left, top);
    end
end

function Shape:SetLeft(left)
    self.baseClass.SetLeft(self, left);
    self:SetProperty("Left", left);
    if (self.overlay) then
        self.overlay:SetLeft(left);
    end
end

function Shape:SetTop(top)
    self.baseClass.SetTop(self, top);
    self:SetProperty("Top", top);
    if (self.overlay) then
        self.overlay:SetTop(top);
    end
end

function Shape:SetZOrder(z)
    self.baseClass.SetZOrder(self, z);
    self:SetProperty("ZOrder", z);
end
