-- Encapsulates data from TerrainMap, by Garan
if (not pcall(import, "GaranStuff.TerrainMap.Grid")) then
    return;
--    error("You must install TerrainMap before using this plugin: https://lotrointerface.com/downloads/info846-TerrainMap.html", 0);
end

_G.TerrainMap = {
    Tiles = _G.GaranStuff.TerrainMap.Grid;
    Region = {
        Eriador = 1;
        Rhovanion = 2;
        Gondor = 3;
        Mordor = 4;
    };

    -- Sign convention for geographic coordinates: Use negative numbers for South and West.
    --        N
    --        ^
    --   -W <   > E
    --        v
    --       -S    
    Origin = { -- top, left
        {  26.0,  -110.75 }; -- Eriador
        {  47.62, -119.6  }; -- Rhovanion
        { -15.6,   -96.4  }; -- Gondor
        { -15.6,   -25.2  }; -- Mordor
    };
};
_G.GaranStuff = nil;

-- Fully qualify the JPG filenames
for plane = 1, 4 do
    for row, columns in pairs(_G.TerrainMap.Tiles[plane]) do
        for column, data in pairs(columns) do
            local asset = data[1];
            if (type(asset) == "string") then
                asset = "GaranStuff/TerrainMap/Tiles/" .. asset;
            end
            columns[column] = asset; -- remove unnecessary table wrapper
        end
    end
end

-- A function for returning a tile located at specific geographic coordinates (if any tile is there)
-- It returns three values: asset, x, y
--    asset is the asset ID ready to be passed to SetBackground(), or nil if no such tile
--    (x, y) is the pixel position within the tile where the specified geographic coordinates are
-- Notes:
--    In geographic coordinates (N, S, E, W), a tile is 0.8 tall and 0.8 wide.
--    In pixels, a tile is 200 by 200.
function _G.TerrainMap.GetTile(plane, northSouth, eastWest)
    local planeTop, planeLeft = unpack(_G.TerrainMap.Origin[plane]);
    local row = math.floor((planeTop - northSouth) / 0.8);
    local column = math.floor((eastWest - planeLeft) / 0.8);
    local asset = _G.TerrainMap.Tiles[plane][row] and _G.TerrainMap.Tiles[plane][row][column];
    local tileTop = planeTop - 0.8 * (row - 1);
    local tileLeft = planeLeft + 0.8 * (column - 1);
    local x = math.floor(0.5 + 200 * (eastWest - tileLeft) / 0.8);
    local y = math.floor(0.5 + 200 * (tileTop - northSouth) / 0.8);
    return asset, x, y;
end
